@@ -14,7 +14,7 @@ from mitosheet.step_performers.graph_steps.graph_utils import (
 )
 from mitosheet.sheet_functions.types.utils import is_number_dtype
 from mitosheet.types import ColumnHeader, ColumnID
-from mitosheet.mito_analytics import log
+from mitosheet.telemetry.telemetry_utils import log
 from mitosheet.steps_manager import StepsManager
 from mitosheet.step_performers.bulk_old_rename.deprecated_utils import deprecated
 from mitosheet.step_performers.graph_steps.graph_utils import BAR, BOX, SCATTER
@@ -24,15 +24,16 @@ from mitosheet.step_performers.graph_steps.graph_utils import BAR, BOX, SCATTER
 MAX_UNIQUE_NON_NUMBER_VALUES = 10_000
 
 
-def get_column_summary_graph(event: Dict[str, Any], steps_manager: StepsManager) -> str:
+def get_column_summary_graph(params: Dict[str, Any], steps_manager: StepsManager) -> str:
     """
     Creates a column summary graph and sends it back as a PNG
     string to the frontend for display.
     """
-    sheet_index = event['sheet_index']
-    column_id: ColumnID = event['column_id']
-    height = event['height']
-    width = event['width']
+    sheet_index = params['sheet_index']
+    column_id: ColumnID = params['column_id']
+    height = params['height']
+    width = params['width']
+    include_plotlyjs = params['include_plotlyjs']
 
 
     # Create a copy of the dataframe, just for safety.
@@ -51,7 +52,7 @@ def get_column_summary_graph(event: Dict[str, Any], steps_manager: StepsManager)
         )
     )
 
-    return_object = get_html_and_script_from_figure(fig, height, width)
+    return_object = get_html_and_script_from_figure(fig, height, width, include_plotlyjs)
 
     return json.dumps(return_object)
 
